package net.wanji.utc.hisense.service.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.common.enums.DataBrainControlModeEnum;
import net.wanji.utc.hisense.common.enums.DataBrainDirEnum;
import net.wanji.utc.hisense.common.enums.DataBrainLampStatusEnum;
import net.wanji.utc.hisense.common.enums.DataBrainTurnEnum;
import net.wanji.utc.hisense.pojo.convert.HisenseLightStatusPojo;
import net.wanji.utc.hisense.service.SignalStatusService;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:28
 */
@Service
@Slf4j
public class SignalStatusServiceImpl implements SignalStatusService {

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) {
        return null;
    }

    @Override
    public List<LightsStatusVO> lightStatus(String crossId) {
        List<LightsStatusVO> lightsStatusVOS = new ArrayList<>();
        LightsStatusVO lightsStatusVO = SignalDataCache.runningStateInfoCache4DataBrain.get(crossId);
        lightsStatusVOS.add(lightsStatusVO);
        return lightsStatusVOS;
    }

    @Override
    public void receiveLightStatus(HisenseLightStatusPojo hisenseLightStatusPojo) throws Exception {
        ObjectMapper jackson = JacksonUtils.getInstance();
        String bodyStr = hisenseLightStatusPojo.getBody();
        List<HisenseLightStatusPojo.Body> bodies = jackson.readValue(bodyStr, new TypeReference<List<HisenseLightStatusPojo.Body>>() {
        });
        if (!CollectionUtils.isEmpty(bodies)) {
            for (HisenseLightStatusPojo.Body body : bodies) {
                HisenseLightStatusPojo.ContentBody content = body.getContent();
                LightsStatusVO lightsStatusVO = new LightsStatusVO();
                String hiCrossId = content.getCrossId();
                CrossInfoPO crossInfo = CrossInfoCache.getCrossInfoBySignalCode(hiCrossId);
                if (Objects.isNull(crossInfo)) {
                    continue;
                }
                String crossId = crossInfo.getId();
                int cycleCountDown = content.getCycleCountDown();
                PriorityQueue<LightsStatusVO> crossPriority = SignalDataCache.crossPriority.get(crossId);
                if (Objects.isNull(crossPriority)) {
                    crossPriority = new PriorityQueue<>((o1, o2) -> {
                        long l = Long.parseLong(o1.getSchemeStartTime()) - Long.parseLong(o2.getSchemeStartTime());
                        return (int) l;
                    });
                } else {
                    if (crossPriority.size() > 0) {
                        LightsStatusVO lightsStatusVOCache = crossPriority.peek();
                        if (Objects.nonNull(lightsStatusVOCache)) {
                            if (lightsStatusVOCache.getCycleCountDown() <= cycleCountDown) {
                                continue;
                            }
                        }
                    }
                }

                int hiControlMode = content.getControlMode();
                int period = content.getPeriod();
                int planId = content.getPlanId();
                Long signalMachineStamp = content.getSignalMachineStamp();
                if (StringUtils.equals(crossId, "13NF80B5QN0")) {
                    log.error("当前路口编号：{}， 周期倒计时：{}，周期时间：{}", crossId, cycleCountDown, period);
                }
                lightsStatusVO.setCrossId(crossId);
                lightsStatusVO.setSchemeId(String.valueOf(planId));
                String runMode = String.valueOf(DataBrainControlModeEnum.getWjControlMode(hiControlMode));
                lightsStatusVO.setRunMode(runMode);
                lightsStatusVO.setCycleCountDown(cycleCountDown);
                lightsStatusVO.setCycleLen(period);
                lightsStatusVO.setSchemeStartTime(String.valueOf(signalMachineStamp));


                // 保证不为空值
                lightsStatusVO.setPhaseMap(Collections.emptyMap());
                lightsStatusVO.setControlMode(runMode);
                lightsStatusVO.setCode(hiCrossId);
                lightsStatusVO.setManufacturerCode("hisense");
                lightsStatusVO.setPhasePlanId(String.valueOf(planId));

                List<HisenseLightStatusPojo.ContentBody.LampRealInfos> lampRealInfos = content.getLampRealInfos();
                if (!CollectionUtils.isEmpty(lampRealInfos)) {
                    Map<Integer, List<HisenseLightStatusPojo.ContentBody.LampRealInfos>> dirLampMap = lampRealInfos.stream().collect(Collectors.groupingBy(HisenseLightStatusPojo.ContentBody.LampRealInfos::getRoadDirection));
                    Map<String, Object> dirMap = new HashMap<>();
                    for (Map.Entry<Integer, List<HisenseLightStatusPojo.ContentBody.LampRealInfos>> dirEntry : dirLampMap.entrySet()) {
                        Integer roadDirection = dirEntry.getKey();
                        List<HisenseLightStatusPojo.ContentBody.LampRealInfos> dirLampList = dirEntry.getValue();
                        Map<String, String> turnMap = new HashMap<>();
                        for (HisenseLightStatusPojo.ContentBody.LampRealInfos lampRealInfo : dirLampList) {
                            String lampStatus = lampRealInfo.getLampStatus();
                            Integer countDown = lampRealInfo.getCountDown();
                            Integer roadFlowDirection = lampRealInfo.getRoadFlowDirection();
                            if (roadFlowDirection == 99) {
                                continue;
                            }
                            if (StringUtils.endsWithIgnoreCase(lampStatus, "23")) {
                                lightsStatusVO.setPhaseId("0");
                                lightsStatusVO.setCyclePhaseCountDown(countDown);
                                break;
                            }
                            turnMap.put(String.valueOf(DataBrainTurnEnum.getWjTurn(roadFlowDirection)), DataBrainLampStatusEnum.getWjLampStatus(lampStatus));
                        }
                        dirMap.put(String.valueOf(DataBrainDirEnum.getWjLightsDir(roadDirection)), turnMap);
                    }
                    lightsStatusVO.setDirLampGroupMap(dirMap);
                    crossPriority.offer(lightsStatusVO);
                    if (crossPriority.size() > 100) {
                        crossPriority.clear();
                    }
                    SignalDataCache.runningStateInfoCache4DataBrain.put(crossId, lightsStatusVO);
                    SignalDataCache.crossPriority.put(crossId, crossPriority);
                }
            }
        }
    }


}
